/** 
 * MySwing: Advanced Swing Utilites 
 * Copyright (C) 2005  Santhosh Kumar T 
 * <p/> 
 * This library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation; either 
 * version 2.1 of the License, or (at your option) any later version. 
 * <p/> 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details. 
 */
package com.brocade.web.secsan;

import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import javax.swing.JFrame;

/**
 * Show a frame as modal window.
 * <br><br>
 * <b>First Trick:</b> When you do Frame.setEnabled(false), the components inside that
 * frame will no longer respond to either mouse clicks or key strokes. This is
 * used to block the input to the owner frame. we add a listener to the given
 * frame, when the frame is shown, we disable the owner frame, and enable it back
 * when given frame is closed.
 * <br><br>
 * <b>Second Trick:</b> We want to make the modal frame always on top of owner frame.
 * The disabled Frames doesn't gain focus. But problem arises when both frames
 * are minimized and user tries to open the owner frame. To achieve this. we add
 * a window listener to owner frame, and when it gets activated, we restore the
 * modal frame and bring it to front.
 * <br><br>
 * <b>Third Trick:</b> When the modal listener is closed, we have to remove the window
 * listeners we have registered.
 * <br><br>
 * <b>Last Trick (important):</b> blocking the showAsFrame(...) to return until the
 * modal frame is closed. Here we block the EDT thread, and start dispatching
 * events ourselves until the modal frame is closed. This is the same trick used
 * by JDialog to block setVisible(....) from returning control until dialog is
 * visible.
 * <br><br>
 * How to use this class:
 * <pre>
 * 	// this method call returns only after the modal frame is closed
 * 	ModalFrameUtil.showAsModal(frameToBeShownAsModal, ownerFrame);
 * </pre>
 */

public class ModalFrameUtil{ 
    static class EventPump implements InvocationHandler { 
        Frame frame; 
 
        public EventPump(Frame frame){ 
            this.frame = frame; 
        } 
 
        public Object invoke(Object proxy, Method method, Object[] args) throws Throwable { 
            return frame.isShowing() ? Boolean.TRUE : Boolean.FALSE; 
        } 
 
        // when the reflection calls in this method has to be 
        // replaced once Sun provides a public API to pump events. 
        public void start() throws Exception{ 
            Class clazz = Class.forName("java.awt.Conditional"); 
            Object conditional = Proxy.newProxyInstance( 
                    clazz.getClassLoader(), 
                    new Class[]{clazz}, 
                    this); 
            Method pumpMethod = Class.forName("java.awt.EventDispatchThread") 
                    .getDeclaredMethod("pumpEvents", new Class[]{clazz}); 
            pumpMethod.setAccessible(true); 
            pumpMethod.invoke(Thread.currentThread(), new Object[]{conditional}); 
        } 
    } 
 
    // show the given frame as modal to the specified owner. 
    // NOTE: this method returns only after the modal frame is closed. 
    public static void showAsModal(final Frame frame, final Frame owner){ 
        frame.addWindowListener(new WindowAdapter(){ 
            public void windowOpened(WindowEvent e){ 
                owner.setEnabled(false); 
            } 
 
            public void windowClosed(WindowEvent e){ 
                owner.setEnabled(true); 
                owner.removeWindowListener(this); 
            } 
        }); 
 
        owner.addWindowListener(new WindowAdapter(){ 
            public void windowActivated(WindowEvent e){ 
                if(frame.isShowing()){ 
                    frame.setExtendedState(JFrame.NORMAL); 
                    frame.toFront(); 
                }else 
                    owner.removeWindowListener(this); 
            } 
        }); 
 
        frame.setVisible(true); 
        try{ 
            new EventPump(frame).start(); 
        } catch(Throwable throwable){ 
            throw new RuntimeException(throwable); 
        } 
    } 
}